// ATB_Enc_Prg_1.c * Created: 10.12.2016 21:51:28 Author : AS  
// Auswertung Drehgeber Peter Dannegger
// ATmega 1284p auf Board 1, 16 MHz, Drehgeber auf P98 auf Port A, NT2,

#define F_CPU 16000000					// CPU clock in Hz
#include "avr/io.h"
#include <avr/interrupt.h>

int8_t dreh = 8;						// Angabe der max Stellungen
int8_t val = 0;							// Startwert der Drehung

#define PHASE_A     (PINA & 1<<PA0)		// Eingang Encoder A  P98
#define PHASE_B     (PINA & 1<<PA1)		// Eingang Encoder B  P98	

volatile int8_t enc_delta;				// -128 ... 127
static int8_t last;

void encode_init( void )
  {
	int8_t new;
	new = 0;
	if( PHASE_A ) new = 3;	
	if( PHASE_B ) new ^= 1;				// convert gray to binary
	last = new;							// power on state
	enc_delta = 0;
  }

ISR( TIMER0_COMPA_vect )				// 1ms for manual movement
  {
	int8_t new, diff;
	new = 0;
	if( PHASE_A ) new = 3;
	if( PHASE_B ) new ^= 1;				// convert gray to binary
	diff = last - new;					// difference last - new
	if( diff & 1 ) 
	  {									// bit 0 = value (1)
		last = new;						// store new as next last
		enc_delta += (diff & 2) - 1;	// bit 1 = direction (+/-)
	  }
  }

int8_t encode_read1( void )			// einzel Schritt Encoder
  {
	int8_t val;
	cli();
	val = enc_delta;
	enc_delta = 0;
	sei();
	return val;                   
  }

int8_t encode_read2( void )			// zwei Schritt Encoder
  {
	int8_t val;
	cli();
	val = enc_delta;
	enc_delta = val & 1;
	sei();
	return val >> 1;
  }

int8_t encode_read4( void )			// vier Schritt Encoder
  {
	int8_t val;
	cli();
	val = enc_delta;
	enc_delta = val & 3;
	sei();
	return val >> 2;
  }

void timer_init()					// Timer 1ms
  {
	TCCR0A = (1<<WGM01);
	TCCR0B = (1<<CS01)|(1<<CS00);
	OCR0A = 249;
	TIMSK0 |=(1<<OCIE0A);
  }
  
int main( void )					// Hauptschleife
  {
	timer_init();  
	DDRA |= 0b11111100 ;			// LED auf Ausgang
	PORTA = 0b01111100;				// LED auf aus bei 1
	encode_init();
	sei();
	while(1)
	  {
		val += encode_read4();		// Auswahl 1,2 oder 4 Schritt Encoder
		if (val>dreh)				// Begrenzung oben
		  {
		    val=0;
		  }   
		if (val<0)					// Begrenzung unten
		  {
			val=dreh;
		  }							// Beginn Auswertung
		if (val==1)					// LED blau
		  {
		    PORTA &= ~(1<<PA4);		// LED Blau ein
		  }
		else
		  {
			PORTA |= (1<<PA4);		// LED Blau aus
		  }
		
		if (val==2)					// LED grn
		  {
			PORTA &= ~(1<<PA3);		// LED grn ein
		  }
		else
		  {
			PORTA |= (1<<PA3);		// LED grn aus
		  }
		if (val==3)					// LED rot
		  {
			PORTA &= ~(1<<PA2);		// LED rot ein
		  }
		else
		  {
			PORTA |= (1<<PA2);		// LED rot aus
		  }
		if (val==5)
		  {
			PORTA &= ~(1<<PA3);		// LED grn ein
			PORTA &= ~(1<<PA4);		// LED blau ein
		  }
		if (val==6)
		  {
			PORTA &= ~(1<<PA2);		// LED rot ein
			PORTA &= ~(1<<PA4);		// LED blau ein
		  }
		if (val==7)
		  {
			PORTA &= ~(1<<PA2);		// LED rot ein
			PORTA &= ~(1<<PA3);		// LED grn ein
		  }
		if (val==8)
		  {
			PORTA &= ~(1<<PA2);		// LED rot ein
			PORTA &= ~(1<<PA3);		// LED grn ein
			PORTA &= ~(1<<PA4);		// LED blau ein	
		  }	
	    }
      }

































